pub struct MetaEntry {
pub visibility: MetaVisibility,
pub value: String,
}Expand description
An entry in the MAF Meta API.
The value stored in the entry should be unmarshalled from JSON using MetaEntry::deserialize.
Fields
visibility: MetaVisibilityvalue: StringImplementations
Sourceimpl MetaEntry
impl MetaEntry
Sourcepub fn deserialize<T>(&self) -> Result<T, Error>where
T: DeserializeOwned,
pub fn deserialize<T>(&self) -> Result<T, Error>where
T: DeserializeOwned,
Deserialize the value of the meta entry into the specified type.
Sourcepub fn visibility(&self) -> &MetaVisibility
pub fn visibility(&self) -> &MetaVisibility
Get the visibility of the meta entry.